==> gatsby-source-netlify-cms-0.2.0.tgz: package/gatsby-node.js <==
"use strict";

const fs = require('fs');

const path = require('path');

const mkdirp = require('mkdirp');

const jsYaml = require('js-yaml');

const toml = require('toml');

const mdTransformer = require('gatsby-transformer-remark/gatsby-node');

const widgetTypes = require('./widget-type-defs');

exports.onCreateNode = async (props, options) => {
  const {
    node,
    createNodeId,
    createContentDigest,
    loadNodeContent,
    actions,
    reporter
  } = props;
  const {
    createNode,
    createParentChildLink
  } = actions;
  const {
    types = {}
  } = options;
  const debug = options.debug ? reporter.info : () => {
    return;
  }; // Only process files that match the types being configured in this plugin

  if (!node.sourceInstanceName || node.internal.type !== "File") return;
  const notPluginType = Boolean(!types[node.sourceInstanceName]);
  debug(`${notPluginType ? 'NOT Processing' : 'Processing'} ${node.internal.type} ${node.sourceInstanceName}`);
  if (notPluginType) return;

  const mappedTypeName = name => {
    if (!types) return name;
    return types[name] ? types[name].type || name : name;
  };

  const createJsonNode = async parse => {
    try {
      const content = await loadNodeContent(node);
      const parsedContent = parse(content);
      const typeName = mappedTypeName(node.sourceInstanceName);
      const jsonNode = Object.assign({}, parsedContent, {
        id: createNodeId(`${node.id} >>> ${typeName}`),
        children: [],
        parent: node.id,
        internal: {
          contentDigest: createContentDigest(parsedContent),
          type: `${typeName}`
        }
      });
      createNode(jsonNode);
      createParentChildLink({
        parent: node,
        child: jsonNode
      });
      debug(`Created Node ${typeName} item: ${jsonNode.id}`);
      return jsonNode;
    } catch (err) {
      reporter.panicOnBuild(`Error processing ${node.sourceInstanceName ? `${mappedTypeName(node.sourceInstanceName)} from file ${node.absolutePath}` : `file ${node.absolutePath}`}:\n
        ${err.message}`);
      return {};
    }
  }; // Ensures we are processing only files from the plugin setup (collections)


  if (node.internal.mediaType === 'application/json') {
    return createJsonNode(JSON.parse);
  }

  if (node.internal.mediaType === 'text/yaml') {
    return createJsonNode(jsYaml.load);
  }

  if (node.extension === 'toml') {
    return createJsonNode(toml.parse);
  }

  if (node.internal.mediaType === `text/markdown` || node.internal.mediaType === `text/x-markdown`) {
    try {
      const markdownNode = await mdTransformer.onCreateNode(props, {});
      const typeName = mappedTypeName(node.sourceInstanceName);
      const newNode = {
        id: createNodeId(`${node.id} >>> ${typeName}`),
        children: [],
        parent: node.id,
        internal: {
          type: `${typeName}`
        },
        rawMarkdownBody: markdownNode.rawMarkdownBody,
        frontmatter: markdownNode.frontmatter,
        fileAbsolutePath: markdownNode.fileAbsolutePath
      };
      newNode.internal.contentDigest = createContentDigest(newNode);
      createNode(newNode);
      createParentChildLink({
        parent: markdownNode,
        child: newNode
      });
      debug(`Created Node ${typeName} item: ${newNode.id}`);
      return newNode;
    } catch (err) {
      reporter.panicOnBuild(`Error processing ${node.sourceInstanceName ? `${mappedTypeName(node.sourceInstanceName)} from file ${node.absolutePath}` : `file ${node.absolutePath}`}:\n
        ${err.message}`);
      return {};
    }
  }
};

exports.createSchemaCustomization = ({
  actions,
  schema,
  reporter
}, options) => {
  const {
    config,
    widgetTypeDefs = {},
    types = {}
  } = options || {};
  const {
    createTypes
  } = actions;
  const debug = options.debug ? reporter.info : () => {
    return;
  };
  /*
    widgetType is an object with widgets as the key
    resolves the widget key name to a graphql type
  */

  const widgetType = Object.assign({}, widgetTypes, {}, options.widgetTypeDefs);

  const getType = field => {
    const {
      widget,
      required = true
    } = field;
    let type = widgetType[widget]; // default

    if (typeof type === 'object') {
      type = type.resolver ? type.resolver(field, type.default) : type.default;
      debug(`${widget} Field: ${JSON.stringify(field, null, 2)}`);
    }

    return type ? `${type}${required ? '!' : ''}` : type;
  };
  /*
    Get mapped name from options types or default
  */


  const mappedName = name => {
    if (!types) return name;
    return types[name] ? types[name].type || name : name;
  };
  /*
    Get options types is node
  */


  const mappedIsNode = name => {
    if (!types) return false;
    return Boolean(types[name] && types[name].isNode);
  };
  /*
    Get options types is node
  */


  const isInferredWidget = widget => Boolean([undefined, 'hidden'].indexOf(widget));

  const mappedIsInferred = (name, widget) => {
    if (!types) return false;
    return Boolean(types[name] && types[name].infer || isInferredWidget(widget));
  };
  /*
    Get typeDefs array for a collection
    if a type has a field of sub type (object), traverse recursively
    Returns an array of typeDefs with subtypes if any
  */


  const getTypeDefs = ({
    name,
    // type name
    fields,
    // config fields to map to graphql types
    isNode,
    // is this a node type?
    infer // should gatsby infer more fields?

  }) => {
    let typeDefs = [];
    const typeFields = {};
    fields.forEach(field => {
      if (field.collection && field.valueField) {
        // Relation widget field. Write a resolver for relation field.
        const relationType = `${mappedName(field.collection)}`;
        debug(`Creating Resolver ${name}.${field.name} of ${Boolean(field.multiple) ? `[${relationType}]` : `${relationType}`}`);
        typeFields[field.name] = {
          type: Boolean(field.multiple) ? `[${relationType}]` : `${relationType}`,

          resolve(parent, args, context) {
            const getValue = (obj, arr) => {
              const field = arr.shift();
              if (typeof field === 'undefined' || obj[field] === undefined) return obj;
              const subObj = obj[field];
              return getValue(subObj, [...arr]);
            };

            const isMultiple = Boolean(field.multiple);
            const valueItem = parent[`${field.name}`]; // A value field can have dot '.' notation, so later we split and getValue

            const valueField = `${field.valueField}`; // get all the nodes for the relation collections type

            const nodes = context.nodeModel.getAllNodes({
              type: `${relationType}`
            });

            if (isMultiple) {
              // Returns an array of the relation type
              return nodes.filter(node => {
                const nodeData = node.frontmatter || node;
                const nodeItem = getValue(nodeData, valueField.split('.'));
                return Boolean(valueItem.indexOf(nodeItem) + 1);
              });
            } else {
              // Returns the first single item found
              return nodes.find(node => {
                const nodeData = node.frontmatter || node;
                const nodeValue = getValue(nodeData, valueField.split('.'));
                return nodeValue === valueItem;
              });
            }
          }

        };
        return;
      }

      const type = getType(field);
      if (!type) return; // ignore this field if widget type not mapped

      if (type.startsWith('SubTypeArrayOfString')) {
        typeFields[field.name] = [`String${field.required ? '!' : ''}`];
      } else if (type.startsWith('SubType')) {
        // Object Field sub type
        const subName = `${name}_${field.name}`;
        const subType = type.startsWith('SubTypeArrayOfObject') ? [`${mappedName(subName)}!`] : `${mappedName(subName)}!`;
        typeFields[field.name] = mappedIsNode(subName) ? {
          type: subType,
          link: {} // Linked SubType (object Node)

        } : subType;
        const subDefs = getTypeDefs({
          name: subName,
          fields: field.fields,
          isNode: mappedIsNode(subName),
          infer: mappedIsInferred(subName, field.widget)
        });
        typeDefs = [...typeDefs, ...subDefs];
      } else {
        // Primitive graphql type
        typeFields[field.name] = type;
      }
    });
    const typeDef = {
      name: `${mappedName(name)}`,
      fields: typeFields,
      extensions: {
        infer: Boolean(infer)
      }
    };
    if (isNode) typeDef['interfaces'] = [`Node`];
    typeDefs.push(schema.buildObjectType(typeDef));
    return typeDefs;
  };
  /*
    typeDefs will contain all collection types (nodes)
  */


  try {
    let typeDefs = [];

    if (config && config.collections) {
      config.collections.forEach(collection => {
        if (collection.folder) {
          // Check if the type is markdown on this collection and wrap:
          // rawMarkdownBody, frontmatter, and fileAbsolutePath fields
          // frontmatter will wrap collection fields from markdown
          const isMarkdown = Boolean(['undefined', undefined, 'md', 'markdown'].indexOf(collection.extension)) || !Boolean(['undefined', undefined, 'yml', 'yaml', 'toml', 'json'].indexOf(collection.format));

          if (isMarkdown) {
            collection.fields = [{
              label: 'frontmatter',
              name: 'frontmatter',
              widget: 'object',
              fields: collection.fields
            }, {
              label: 'rawMarkdownBody',
              name: 'rawMarkdownBody',
              widget: 'string'
            }, {
              label: 'fileAbsolutePath',
              name: 'fileAbsolutePath',
              widget: 'string'
            }];
          }

          const defs = getTypeDefs({
            name: collection.name,
            fields: collection.fields,
            isNode: true,
            infer: mappedIsInferred(collection.name)
          });
          typeDefs = [...typeDefs, ...defs];
        } else {
          // Files collection (node with subTypes)
          const typeFields = {};
          collection.files.forEach(file => {
            const name = `${collection.name}_${file.name}`;
            const defs = getTypeDefs({
              name,
              fields: file.fields,
              isNode: true,
              // file types are always nodes
              infer: mappedIsInferred(name)
            });
            typeDefs = [...typeDefs, ...defs];
          });
        }
      });
    }

    createTypes(typeDefs);
    debug(`typeDefs: ${JSON.stringify(typeDefs, null, 2)}`);
  } catch (error) {
    reporter.panic('[gatsby-source-netlify-cms] Error buiding types', error);
  }
};
/*
  Runs prior to Gatsby doing anything else
  We are creating the content path if it does not exist!
  So, dynamic and failsafe! Or at least, we'll try.
*/


exports.onPreBootstrap = (props, options) => {
  const {
    store,
    reporter
  } = props;
  const {
    program
  } = store.getState();
  const {
    config
  } = options || {};

  if (typeof config !== 'object') {
    reporter.panic('[gatsby-source-netlify-cms] options.config must be a json object');
  }

  if (!Array.isArray(config.collections)) {
    reporter.panic('[gatsby-source-netlify-cms] collections missing from config');
  }

  const debug = options.debug ? reporter.info : () => {
    return;
  };
  /*
    Confirm or create media_folder directory
    if it exists in the config
  */

  if (config.media_folder) {
    const mediaDir = path.join(program.directory, config.media_folder);

    if (!fs.existsSync(mediaDir)) {
      mkdirp.sync(mediaDir);
      debug(`Created: ${mediaDir}`);
    }
  }
  /*
    Confirm or create collections file/files directories
  */


  config.collections.forEach(collection => {
    if (collection.folder) {
      const dir = path.join(program.directory, collection.folder);

      if (!fs.existsSync(dir)) {
        mkdirp.sync(dir);
        debug(`Folder Created: ${dir}`);
      }
    } else {
      collection.files.forEach(item => {
        const filePath = path.join(program.directory, item.file);
        const dir = path.dirname(filePath);

        if (!fs.existsSync(dir)) {
          mkdirp.sync(dir);
          debug(`File Directory Created: ${dir}`);
        }

        if (!fs.existsSync(filePath)) {
          fs.writeFileSync(filePath, '', {
            flag: 'w',
            encoding: 'utf8'
          });
          debug(`Empty File Created: ${filePath}`);
        }
      });
    }
  });
};

==> gatsby-source-netlify-cms-0.4.0.tgz: package/gatsby-node.js <==
"use strict";

const fs = require('fs');

const path = require('path');

const mkdirp = require('mkdirp');

const jsYaml = require('js-yaml');

const toml = require('toml');

const mdTransformer = require('gatsby-transformer-remark/gatsby-node');

const widgetTypes = require('./widget-type-defs');

exports.onCreateNode = async (props, options) => {
  const {
    node,
    createNodeId,
    createContentDigest,
    loadNodeContent,
    actions,
    reporter
  } = props;
  const {
    createNode,
    createParentChildLink
  } = actions;
  const {
    types = {}
  } = options;
  const debug = options.debug ? reporter.info : () => {
    return;
  }; // Only process files that match the types being configured in this plugin

  if (!node.sourceInstanceName || node.internal.type !== "File") return;
  const notPluginType = Boolean(!types[node.sourceInstanceName]);
  debug(`${notPluginType ? 'NOT Processing' : 'Processing'} ${node.internal.type} ${node.sourceInstanceName}`);
  if (notPluginType) return;

  const mappedTypeName = name => {
    if (!types) return name;
    return types[name] ? types[name].type || name : name;
  };

  const createJsonNode = async parse => {
    try {
      const content = await loadNodeContent(node);
      const parsedContent = parse(content);
      const typeName = mappedTypeName(node.sourceInstanceName);
      const jsonNode = Object.assign({}, parsedContent, {
        id: createNodeId(`${node.id} >>> ${typeName}`),
        children: [],
        parent: node.id,
        internal: {
          contentDigest: createContentDigest(parsedContent),
          type: `${typeName}`
        }
      });
      createNode(jsonNode);
      createParentChildLink({
        parent: node,
        child: jsonNode
      });
      debug(`Created Node ${typeName} item: ${jsonNode.id}`);
      return jsonNode;
    } catch (err) {
      reporter.panicOnBuild(`Error processing ${node.sourceInstanceName ? `${mappedTypeName(node.sourceInstanceName)} from file ${node.absolutePath}` : `file ${node.absolutePath}`}:\n
        ${err.message}`);
      return {};
    }
  }; // Ensures we are processing only files from the plugin setup (collections)


  if (node.internal.mediaType === 'application/json') {
    return createJsonNode(JSON.parse);
  }

  if (node.internal.mediaType === 'text/yaml') {
    return createJsonNode(jsYaml.load);
  }

  if (node.extension === 'toml') {
    return createJsonNode(toml.parse);
  }

  if (node.internal.mediaType === `text/markdown` || node.internal.mediaType === `text/x-markdown`) {
    try {
      const markdownNode = await mdTransformer.onCreateNode(props, {});
      const typeName = mappedTypeName(node.sourceInstanceName);
      const newNode = {
        id: createNodeId(`${node.id} >>> ${typeName}`),
        children: [],
        parent: node.id,
        internal: {
          type: `${typeName}`
        },
        body: markdownNode.rawMarkdownBody,
        frontmatter: markdownNode.frontmatter,
        fileAbsolutePath: markdownNode.fileAbsolutePath
      };
      newNode.internal.contentDigest = createContentDigest(newNode);
      createNode(newNode);
      createParentChildLink({
        parent: markdownNode,
        child: newNode
      });
      debug(`Created Node ${typeName} item: ${newNode.id}`);
      return newNode;
    } catch (err) {
      reporter.panicOnBuild(`Error processing ${node.sourceInstanceName ? `${mappedTypeName(node.sourceInstanceName)} from file ${node.absolutePath}` : `file ${node.absolutePath}`}:\n
        ${err.message}`);
      return {};
    }
  }
};

exports.createSchemaCustomization = ({
  actions,
  schema,
  reporter
}, options) => {
  const {
    config,
    widgetTypeDefs = {},
    types = {}
  } = options || {};
  const {
    createTypes
  } = actions;
  const debug = options.debug ? reporter.info : () => {
    return;
  };
  /*
    widgetType is an object with widgets as the key
    resolves the widget key name to a graphql type
  */

  const widgetType = Object.assign({}, widgetTypes, {}, options.widgetTypeDefs);

  const getType = field => {
    const {
      widget,
      required = true
    } = field;
    let type = widgetType[widget]; // default

    if (typeof type === 'object') {
      type = type.resolver ? type.resolver(field, type.default) : type.default;
      debug(`${widget} Field: ${JSON.stringify(field, null, 2)}`);
    }

    return type ? `${type}${required ? '!' : ''}` : type;
  };
  /*
    Get mapped name from options types or default
  */


  const mappedName = name => {
    if (!types) return name;
    return types[name] ? types[name].type || name : name;
  };
  /*
    Get options types is node
  */


  const mappedIsNode = name => {
    if (!types) return false;
    return Boolean(types[name] && types[name].isNode);
  };
  /*
    Get options types is node
  */


  const isInferredWidget = widget => Boolean([undefined, 'hidden'].indexOf(widget));

  const mappedIsInferred = (name, widget) => {
    if (!types) return false;
    return Boolean(types[name] && types[name].infer || isInferredWidget(widget));
  };
  /*
    Get typeDefs array for a collection
    if a type has a field of sub type (object), traverse recursively
    Returns an array of typeDefs with subtypes if any
  */


  const getTypeDefs = ({
    name,
    // type name
    fields,
    // config fields to map to graphql types
    isNode,
    // is this a node type?
    infer // should gatsby infer more fields?

  }) => {
    let typeDefs = [];
    const typeFields = {};
    fields.forEach(field => {
      if (field.collection && field.valueField) {
        // Relation widget field. Write a resolver for relation field.
        const relationType = `${mappedName(field.collection)}`;
        debug(`Creating Resolver ${name}.${field.name} of ${Boolean(field.multiple) ? `[${relationType}]` : `${relationType}`}`);
        typeFields[field.name] = {
          type: Boolean(field.multiple) ? `[${relationType}]` : `${relationType}`,

          resolve(parent, args, context) {
            const getValue = (obj, arr) => {
              const field = arr.shift();
              if (typeof field === 'undefined' || obj[field] === undefined) return obj;
              const subObj = obj[field];
              return getValue(subObj, [...arr]);
            };

            const isMultiple = Boolean(field.multiple);
            const valueItem = parent[`${field.name}`]; // A value field can have dot '.' notation, so later we split and getValue

            const valueField = `${field.valueField}`; // get all the nodes for the relation collections type

            const nodes = context.nodeModel.getAllNodes({
              type: `${relationType}`
            });

            if (isMultiple) {
              // Returns an array of the relation type
              return nodes.filter(node => {
                const nodeData = node.frontmatter || node;
                const nodeItem = getValue(nodeData, valueField.split('.'));
                return Boolean(valueItem.indexOf(nodeItem) + 1);
              });
            } else {
              // Returns the first single item found
              return nodes.find(node => {
                const nodeData = node.frontmatter || node;
                const nodeValue = getValue(nodeData, valueField.split('.'));
                return nodeValue === valueItem;
              });
            }
          }

        };
        return;
      }

      const type = getType(field);
      if (!type) return; // ignore this field if widget type not mapped

      if (type.startsWith('SubTypeArrayOfString')) {
        typeFields[field.name] = [`String${field.required ? '!' : ''}`];
      } else if (type.startsWith('SubType')) {
        // Object Field sub type
        const subName = `${name}_${field.name}`;
        const subType = type.startsWith('SubTypeArrayOfObject') ? [`${mappedName(subName)}!`] : `${mappedName(subName)}!`;
        typeFields[field.name] = mappedIsNode(subName) ? {
          type: subType,
          link: {} // Linked SubType (object Node)

        } : subType;
        const subDefs = getTypeDefs({
          name: subName,
          fields: field.fields,
          isNode: mappedIsNode(subName),
          infer: mappedIsInferred(subName, field.widget)
        });
        typeDefs = [...typeDefs, ...subDefs];
      } else {
        // Primitive graphql type
        typeFields[field.name] = type;
      }
    });
    const typeDef = {
      name: `${mappedName(name)}`,
      fields: typeFields,
      extensions: {
        infer: Boolean(infer)
      }
    };
    if (isNode) typeDef['interfaces'] = [`Node`];
    typeDefs.push(schema.buildObjectType(typeDef));
    return typeDefs;
  };
  /*
    typeDefs will contain all collection types (nodes)
  */


  try {
    let typeDefs = [];

    if (config && config.collections) {
      config.collections.forEach(collection => {
        if (collection.folder) {
          // Check if the type is markdown on this collection and wrap:
          // rawMarkdownBody, frontmatter, and fileAbsolutePath fields
          // frontmatter will wrap collection fields from markdown
          const isMarkdown = Boolean(['undefined', undefined, 'md', 'markdown'].indexOf(collection.extension)) || !Boolean(['undefined', undefined, 'yml', 'yaml', 'toml', 'json'].indexOf(collection.format));

          if (isMarkdown) {
            // Hoist body field up from frontmatter fields
            const frontmatterFields = collection.fields.filter(field => field.name !== 'body');
            collection.fields = [{
              label: 'frontmatter',
              name: 'frontmatter',
              widget: 'object',
              fields: frontmatterFields
            }, {
              label: 'rawMarkdownBody',
              name: 'body',
              widget: 'string'
            }, {
              label: 'fileAbsolutePath',
              name: 'fileAbsolutePath',
              widget: 'string'
            }];
          }

          const defs = getTypeDefs({
            name: collection.name,
            fields: collection.fields,
            isNode: true,
            infer: mappedIsInferred(collection.name)
          });
          typeDefs = [...typeDefs, ...defs];
        } else {
          // Files collection (node with subTypes)
          const typeFields = {};
          collection.files.forEach(file => {
            const name = `${collection.name}_${file.name}`;
            const defs = getTypeDefs({
              name,
              fields: file.fields,
              isNode: true,
              // file types are always nodes
              infer: mappedIsInferred(name)
            });
            typeDefs = [...typeDefs, ...defs];
          });
        }
      });
    }

    createTypes(typeDefs);
    debug(`typeDefs: ${JSON.stringify(typeDefs, null, 2)}`);
  } catch (error) {
    reporter.panic('[gatsby-source-netlify-cms] Error buiding types', error);
  }
};
/*
  Runs prior to Gatsby doing anything else
  We are creating the content path if it does not exist!
  So, dynamic and failsafe! Or at least, we'll try.
*/


exports.onPreBootstrap = (props, options) => {
  const {
    store,
    reporter
  } = props;
  const {
    program
  } = store.getState();
  const {
    config
  } = options || {};

  if (typeof config !== 'object') {
    reporter.panic('[gatsby-source-netlify-cms] options.config must be a json object');
  }

  if (!Array.isArray(config.collections)) {
    reporter.panic('[gatsby-source-netlify-cms] collections missing from config');
  }

  const debug = options.debug ? reporter.info : () => {
    return;
  };
  /*
    Confirm or create media_folder directory
    if it exists in the config
  */

  if (config.media_folder) {
    const mediaDir = path.join(program.directory, config.media_folder);

    if (!fs.existsSync(mediaDir)) {
      mkdirp.sync(mediaDir);
      debug(`Created: ${mediaDir}`);
    }
  }
  /*
    Confirm or create collections file/files directories
  */


  config.collections.forEach(collection => {
    if (collection.folder) {
      const dir = path.join(program.directory, collection.folder);

      if (!fs.existsSync(dir)) {
        mkdirp.sync(dir);
        debug(`Folder Created: ${dir}`);
      }
    } else {
      collection.files.forEach(item => {
        const filePath = path.join(program.directory, item.file);
        const dir = path.dirname(filePath);

        if (!fs.existsSync(dir)) {
          mkdirp.sync(dir);
          debug(`File Directory Created: ${dir}`);
        }

        if (!fs.existsSync(filePath)) {
          fs.writeFileSync(filePath, '', {
            flag: 'w',
            encoding: 'utf8'
          });
          debug(`Empty File Created: ${filePath}`);
        }
      });
    }
  });
};
